#include <stdio.h>
#include <math.h>
#include <time.h>

#define MAXDEG 100
#define EPSILON 1e-13*MAXDEG/2
#define MAXITER 5000
#define GLOBAL_NROOTS

#ifdef GLOBAL_NROOTS
int global_nroots;
#endif

typedef struct __tagpoly
{
    int deg;
    double coef[MAXDEG+2];
    int nroots;
    double roots[MAXDEG*2];
} poly;

void poly_init(poly *p);
void poly_print(poly *p);
void poly_printroots(poly *p);
void poly_read(poly *p);
void poly_corr(poly *p);
void poly_norm(poly *p);
double poly_eval(poly *p, double t);
double poly_height(poly *p);
void poly_deriv(poly *p, poly *dpdt);
void poly_rootsN(poly *p, poly *d1p, poly *d2p);
void poly_roots1(poly *p);
void poly_roots2(poly *p, poly *dp1);
double poly_dicho(poly *p, double ta, double tb);

char __dummy001[1024] = "dummydummydummydummydummydummydummydummy";
poly ps[MAXDEG+2];
char __dummy002[1024] = "dummydummydummydummydummydummydummydummy";
FILE *fin, *fout;

void poly_init(poly *p)
{
    int i;
    p->deg = p->nroots = 0;
    for(i=0;i<=MAXDEG+1;i++)
        p->coef[i] = p->roots[i] = .0;
}

void poly_print(poly *p)
{
    int i;
    for(i=p->deg; i>=0; i--)
        fprintf(fout, "x^%d*%lg%s", i, p->coef[i], i?" + ":"");
    fputs("\n", fout);
}

void poly_printroots(poly *p)
{
    int i;
    for(i=0; i<p->nroots; i++)
        fprintf(fout, "x(%d)=%.15lg%s", i, p->roots[i], (i < p->nroots-1)?", ":"");
    fputs("\n", fout);
}

void poly_read(poly *p)
{
    int i;
    fscanf(fin, "%d", &i);
    if((i < 1) || (i > MAXDEG))
    {
        fprintf(fout, "!!!Incorrect degree, must be 1 to %d\n", MAXDEG);
        p->deg = 0;
        p->coef[0] = 0;
        return;
    }
    p->deg = i;
    for(;i>=0;i--)
        fscanf(fin, "%lf", &(p->coef[i]));
}

void poly_corr(poly *p)
{
    while(p->deg && (p->coef[p->deg] == .0))
        p->deg--;
}

void poly_norm(poly *p)
{
    int i;
    if(!p->deg || (p->coef[p->deg] == .0))
        return;
    for(i=0; i<=p->deg; i++)
        p->coef[i] /= p->coef[p->deg];
}

double poly_eval(poly *p, double t)
{
    int i;
    double res = p->coef[p->deg];
    for(i=p->deg-1; i>=0; i--)
    {
        res *= t;
        res += p->coef[i];
    }
    return res;
}

double poly_height(poly *p)
{
    int i;
    double res = .0;
    for(i=0; i<=p->deg; i++)
    {
        res = fabs(p->coef[i])>res?fabs(p->coef[i]):res;
    }
    return res;
}

void poly_deriv(poly *p, poly *dpdt)
{
    int i;
    if(p->deg == 0)
    {
        dpdt->deg = 0;
        dpdt->coef[0] = .0;
    }
    dpdt->deg = p->deg-1;
    for(i=1; i<=p->deg; i++)
        dpdt->coef[i-1] = i*p->coef[i];
}

void poly_roots1(poly *p)
{
    if(p->deg != 1)
        return;
    p->roots[p->nroots++] = -p->coef[0]/p->coef[1];
#ifdef GLOBAL_NROOTS
	++global_nroots;
#endif
}

void poly_roots2(poly *p, poly *d1p)
{
    double d, h;
    if(p->deg != 2)
        return;
    d = p->coef[1]*p->coef[1]/4 - p->coef[0]*p->coef[2];
    h = poly_height(p)*EPSILON;
    
    if(d==0)
    {
        if(fabs(poly_eval(p, d1p->roots[0])) < h)
            p->roots[p->nroots++];
#ifdef GLOBAL_NROOTS
		++global_nroots;
#endif
    }
    else if(d > 0)
    {
        p->roots[p->nroots++] = (-p->coef[1]/2 - sqrt(d))/p->coef[2];
#ifdef GLOBAL_NROOTS
		++global_nroots;
#endif
        p->roots[p->nroots++] = (-p->coef[1]/2 + sqrt(d))/p->coef[2];
#ifdef GLOBAL_NROOTS
		++global_nroots;
#endif
    }
}

void poly_rootsN(poly *p, poly *d1p, poly *d2p)
{
    int i;
    double xmin_, xmax_, x, f0, f, df, h;
    double xmin = 0, xmax = 0;
    h = poly_height(p)*EPSILON;

    /* Newton fork -oo..xmin */
    if(d1p->nroots)
        if(d1p->roots[0] < xmin)
            xmin = d1p->roots[0];
    if(d2p->nroots)
        if(d2p->roots[0] < xmin)
            xmin = d2p->roots[0];
    xmin_ = xmin - 1;
    f0 = poly_eval(p, xmin);
    f = poly_eval(p, xmin_);
    if(f0*f < 0)
	{
        p->roots[p->nroots++] = poly_dicho(p, xmin_, xmin);
#ifdef GLOBAL_NROOTS
		++global_nroots;
#endif
	}
    else
    {
        df = poly_eval(d1p, xmin_);
        x = - f/df - xmin_;
        if((poly_eval(p, x)*f < 0) && (x < xmin_))
		{
            p->roots[p->nroots++] = poly_dicho(p, x, xmin_);
#ifdef GLOBAL_NROOTS
			++global_nroots;
#endif
		}
    }
    /* Inner roots */
    for(i=0;i<d1p->nroots;i++)
    {
        f0 = poly_eval(p, d1p->roots[i]);
        if(fabs(f0) < h)
		{
            p->roots[p->nroots++] = d1p->roots[i];
#ifdef GLOBAL_NROOTS
			++global_nroots;
#endif
		}
        else if(i<d1p->nroots-1)
        {
            f = poly_eval(p, d1p->roots[i+1]);
            if(fabs(f) < h)
                continue;
            else if(f*f0 < 0)
            {
                /* Do dichotomy */
                p->roots[p->nroots++] = poly_dicho(p, d1p->roots[i], d1p->roots[i+1]);
#ifdef GLOBAL_NROOTS
				++global_nroots;
#endif
            }
        }
    }
    /* Newton fork xmax..+oo */
    if(d1p->nroots)
        if(d1p->roots[d1p->nroots-1] > xmax)
            xmax = d1p->roots[d1p->nroots-1];
    if(d2p->nroots)
        if(d2p->roots[d2p->nroots-1] > xmax)
            xmax = d2p->roots[d2p->nroots-1];
    xmax_ = xmax + 1;
    f0 = poly_eval(p, xmax);
    f = poly_eval(p, xmax_);
    if(f0*f < 0)
	{
        p->roots[p->nroots++] = poly_dicho(p, xmax, xmax_);
#ifdef GLOBAL_NROOTS
		++global_nroots;
#endif
	}
    else
    {
        df = poly_eval(d1p, xmax_);
        x = - f/df - xmax_;
        if((poly_eval(p, x)*f < 0) && (x > xmax_))
		{
            p->roots[p->nroots++] = poly_dicho(p, xmax_, x);
#ifdef GLOBAL_NROOTS
			++global_nroots;
#endif
		}
    }
}

double poly_dicho(poly *p, double ta, double tb)
{
    int count;
    double t;
    double h = poly_height(p)*EPSILON;
    if(ta == tb)
    {
        if(fabs(poly_eval(p, ta)) < h)
            return ta;
        else
        {
            fprintf(fout, "!!! dichotomy incorrect:\n");
            poly_print(p);
            fprintf(fout, "ta=%lg; tb=%lg\n", ta, tb);
            return .0;
        }
    }
    else
    {
        double f, fa = poly_eval(p, ta), fb = poly_eval(p, tb);
        if(fa*fb > 0)
        {
            fprintf(fout, "!!! dichotomy incorrect:\n");
            poly_print(p);
            fprintf(fout, "ta=%lg, val=%lg; tb=%lg, val=%lg\n", ta, fa, tb, fb);
            return .0;
        }

        if((fb < 0) || (fa > 0))
        {
            f = fa; fa = fb; fb = f;
            t = ta; ta = tb; tb = t;
        }

        t = (ta+tb)/2;
        count = 0;

        while((ta != t) && (tb !=t) && (++count < MAXITER))
        {
            f = poly_eval(p, t);
            if(f>0)
            {
                tb =  t;
                fb = f;
            }
            else if(f<0)
            {
                ta =  t;
                fa = f;
            }
            else
                ta = tb = t;
            t = (ta+tb)/2;
        }
        if(count >= MAXITER)
        {
            fprintf(fout, "!!! dichotomy loop:\n");
            poly_print(p);
            fprintf(fout, "ta=%lg, val=%lg; tb=%lg, val=%lg, >%d iterations\n", ta, fa, tb, fb, count);
            return .0;
        }
        return (fabs(fa) < fabs(fb))?ta:tb;
    }
}

int main(int argc, char * argv[])
{
    int i;
    time_t t0, t1;

    fin = fopen("in.txt", "rt");
    fout = fopen("out.txt", "wt");

    ps[0].deg = 3;

    t0 = time(0);

#ifdef GLOBAL_NROOTS
	global_nroots = 0;
#endif
    while(ps[0].deg > 1)
    {
        fprintf(fout, "========================================\n");

        for(i=0; i<=MAXDEG+1; i++)
            poly_init(ps+i);

        poly_read(ps);
        if(!ps[0].deg)
            continue;

        poly_corr(ps);
        poly_norm(ps);

        for(i=0; i<ps[0].deg-1; i++)
        {
            poly_deriv(ps+i, ps+i+1);
            poly_norm(ps+i+1);
        }

        poly_roots1(ps+i);
        fprintf(fout, "%d: ", i);
        poly_printroots(ps+i);

        if(ps[0].deg > 1)
        {
            poly_roots2(ps+i-1, ps+i);
            fprintf(fout, "%d: ", i-1);
            poly_printroots(ps+i-1);
        }

        for(i-=2;i>=0;i--)
        {
            poly_rootsN(ps+i, ps+i+1, ps+i+2);
            fprintf(fout, "%d: ", i);
            poly_printroots(ps+i);
        }
    }

    t1 = time(0);

#ifdef GLOBAL_NROOTS
    fprintf(fout, "%d seconds for %d roots\n", t1-t0, global_nroots);
#else
    fprintf(fout, "%d seconds\n", t1-t0);
#endif

    fclose(fout);
    fclose(fin);
    return 0;
}
